'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var module$1 = require('module');
var path = require('path');
var vm = require('vm');
var nanoid = require('nanoid');
var url = require('url');
var esbuild = require('esbuild');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

function _interopNamespace(e) {
    if (e && e.__esModule) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () {
                        return e[k];
                    }
                });
            }
        });
    }
    n['default'] = e;
    return Object.freeze(n);
}

var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var vm__default = /*#__PURE__*/_interopDefaultLegacy(vm);

const isInESModuleScope = () => {
    try {
        return module === undefined;
    }
    catch {
        return true;
    }
};
const getCallerDirname = () => {
    const _prepareStackTrace = Error.prepareStackTrace;
    Error.prepareStackTrace = (_err, stack) => stack;
    const stack = new Error().stack.slice(2);
    Error.prepareStackTrace = _prepareStackTrace;
    const fileName = stack[0].getFileName();
    return fileName !== null
        ? path__default['default'].dirname(fileName.startsWith('file://') ? url.fileURLToPath(new URL(fileName)) : fileName)
        : null;
};
const getEntryDirname = () => path__default['default'].dirname(process.argv[1]);

const requireFromString = (code, { dirname, globals } = {}) => {
    var _a, _b, _c;
    const mainModule = isInESModuleScope() ? undefined : require.main;
    const moduleDirname = (_b = (_a = dirname !== null && dirname !== void 0 ? dirname : mainModule === null || mainModule === void 0 ? void 0 : mainModule.path) !== null && _a !== void 0 ? _a : getCallerDirname()) !== null && _b !== void 0 ? _b : getEntryDirname();
    const moduleFilename = path__default['default'].join(moduleDirname, `${nanoid.nanoid()}.js`);
    const contextModule = new module$1.Module(moduleFilename, mainModule);
    contextModule.filename = moduleFilename;
    contextModule.path = moduleDirname;
    contextModule.paths = (_c = mainModule === null || mainModule === void 0 ? void 0 : mainModule.paths) !== null && _c !== void 0 ? _c : [];
    contextModule.require = module$1.createRequire(moduleFilename);
    vm__default['default'].runInNewContext(code, {
        __dirname: contextModule.path,
        __filename: contextModule.filename,
        exports: contextModule.exports,
        module: contextModule,
        require: contextModule.require,
        ...globals
    });
    contextModule.loaded = true;
    return contextModule.exports;
};

const importFromString = async (code, { dirname, globals, transformOptions } = {}) => {
    var _a;
    if (!isInESModuleScope()) {
        ({ code } = await esbuild.transform(code, {
            format: 'cjs',
            ...transformOptions
        }));
        return requireFromString(code, { dirname, globals });
    }
    // @ts-expect-error: experimental
    if (vm__default['default'].Module === undefined) {
        throw new Error('command flag `--experimental-vm-modules` is not enabled');
    }
    if (transformOptions !== undefined) {
        ({ code } = await esbuild.transform(code, {
            format: 'esm',
            ...transformOptions
        }));
    }
    const moduleDirname = (_a = dirname !== null && dirname !== void 0 ? dirname : getCallerDirname()) !== null && _a !== void 0 ? _a : getEntryDirname();
    const moduleFilename = path__default['default'].join(moduleDirname, `${nanoid.nanoid()}.js`);
    const context = vm__default['default'].createContext({
        __IMPORTS__: {},
        ...globals
    });
    // @ts-expect-error: experimental
    const vmModule = new vm__default['default'].SourceTextModule(code, {
        identifier: moduleFilename,
        context
    });
    // @ts-expect-error: experimental
    const linker = async (specifier) => {
        const importedModuleFilename = new RegExp(`^[.\\${path__default['default'].sep}]`).test(specifier)
            ? path__default['default'].resolve(moduleDirname, specifier)
            : undefined;
        const importedModule = await Promise.resolve().then(function () { return /*#__PURE__*/_interopNamespace(require(importedModuleFilename !== null && importedModuleFilename !== void 0 ? importedModuleFilename : specifier)); });
        context.__IMPORTS__[specifier] = importedModule;
        const exportedNames = new Set(Object.getOwnPropertyNames(importedModule));
        const importedModuleContent = `${exportedNames.delete('default') ? `export default __IMPORTS__['${specifier}'].default\n` : ''}export const { ${[...exportedNames].join(', ')} } = __IMPORTS__['${specifier}']`;
        // @ts-expect-error: experimental
        return new vm__default['default'].SourceTextModule(importedModuleContent, {
            identifier: importedModuleFilename !== null && importedModuleFilename !== void 0 ? importedModuleFilename : specifier,
            context
        });
    };
    await vmModule.link(linker);
    await vmModule.evaluate();
    return vmModule.namespace;
};
const importFromStringSync = (code, { dirname, globals, transformOptions } = {}) => {
    if (isInESModuleScope()) {
        throw new Error(`function \`importFromStringSync\` can not work in ES module scope
Use asynchronous function \`importFromString\` instead and execute node with \`--experimental-vm-modules\` command flag.`);
    }
    ({ code } = esbuild.transformSync(code, {
        format: 'cjs',
        ...transformOptions
    }));
    return requireFromString(code, { dirname, globals });
};

exports.importFromString = importFromString;
exports.importFromStringSync = importFromStringSync;
exports.requireFromString = requireFromString;
